//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_GAMEBOYJOYPAD_HH
#define GLBOY_GAMEBOYJOYPAD_HH

#include "glBoy.hh"
#include "Core.hh"

#include "MemoryMap.hh"

#include <SDL/SDL.h>

namespace glBoy
{

class GameboyJoypad : public MemoryMap::Memory
{
public:
	GameboyJoypad(Core& core);

	void pollInput(Core::CallbackId id);

	virtual uint8_t read8(uint16_t address);
	virtual void write8(uint16_t address, uint8_t value);

private:
	enum State { Joypad_Off, Joypad_Directional, Joypad_Button };
	State m_state;

	struct Input
	{
		Input() : sdlKey(0), depressed(false) {}
		int sdlKey;
		bool depressed;
	};

	enum Inputs { UP = 0, DOWN, LEFT, RIGHT, A, B, START, SELECT };
	Input m_inputs[8];

	uint8_t m_buttonState;
	uint8_t m_directionalState;
};

} // namespace glBoy

#endif
