// Instruction: LD r,r'
// Clock: 4
LD_r_r_:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 3) & (uint8_t(-1) >> (8 - 3)) ]]);
	uint8_t& r2(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD r,r'" << " (r=" << r << ")" << std::endl;
#endif


	r = r2;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LD r,n
// Clock: 7
LD_r_n:
{
	uint8_t n(op8());
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 3) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD r,n" << " (n=" << n << ")" << " (r=" << r << ")" << std::endl;
#endif


	r = n;
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD r,(HL)
// Clock: 7
LD_r__HL_:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 3) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD r,(HL)" << " (r=" << r << ")" << std::endl;
#endif


	r = m_mem.read8(m_reg.HL);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD (HL),r
// Clock: 7
LD__HL__r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (HL),r" << " (r=" << r << ")" << std::endl;
#endif


	m_mem.write8(m_reg.HL, r);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD (HL),n
// Clock: 10
LD__HL__n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (HL),n" << " (n=" << n << ")" << std::endl;
#endif


	m_mem.write8(m_reg.HL, n);
	clock += 10;

}
goto END_INSTRUCTIONS;


// Instruction: LD A,(BC)
// Clock: 7
LD_A__BC_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A,(BC)" << std::endl;
#endif


	m_reg.A = m_mem.read8(m_reg.BC);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD A,(DE)
// Clock: 7
LD_A__DE_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A,(DE)" << std::endl;
#endif


	m_reg.A = m_mem.read8(m_reg.DE);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD (BC),A
// Clock: 7
LD__BC__A:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (BC),A" << std::endl;
#endif


	m_mem.write8(m_reg.BC, m_reg.A);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD (DE),A
// Clock: 7
LD__DE__A:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (DE),A" << std::endl;
#endif


	m_mem.write8(m_reg.DE, m_reg.A);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: LD dd,nn
// Clock: 10
LD_dd_nn:
{
	DWORD N(op16());
	DWORD& d(m_reg.begin16[regMask_d[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD dd,nn" << " (N=" << N << ")" << " (d=" << d << ")" << std::endl;
#endif


	d = N;
	clock += 10;

}
goto END_INSTRUCTIONS;


// Instruction: LD SP,HL
// Clock: 6
LD_SP_HL:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD SP,HL" << std::endl;
#endif


	m_reg.SP = m_reg.HL;
	clock += 6;

}
goto END_INSTRUCTIONS;


// Instruction: push qq
// Clock: 11
push_qq:
{
	DWORD& q(m_reg.begin16[regMask_q[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " push qq" << " (q=" << q << ")" << std::endl;
#endif


	m_reg.SP.dec(2);
	m_mem.write16(m_reg.SP, q);
	clock += 11;

}
goto END_INSTRUCTIONS;


// Instruction: pop qq
// Clock: 10
pop_qq:
{
	DWORD& q(m_reg.begin16[regMask_q[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " pop qq" << " (q=" << q << ")" << std::endl;
#endif


	q = m_mem.read16(m_reg.SP);
	m_reg.SP.inc(2);
	clock += 10;

}
goto END_INSTRUCTIONS;


// Instruction: ADD A,r
// Clock: 4
ADD_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADD A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::add(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: ADD A,n
// Clock: 7
ADD_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADD A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::add(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: ADD A,(HL)
// Clock: 7
ADD_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADD A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::add(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: ADC A,r
// Clock: 4
ADC_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADC A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::adc(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: ADC A,n
// Clock: 7
ADC_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADC A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::adc(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: ADC A,(HL)
// Clock: 7
ADC_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADC A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::adc(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: SUB A,r
// Clock: 4
SUB_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SUB A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::sub(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: SUB A,n
// Clock: 7
SUB_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SUB A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::sub(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: SUB A,(HL)
// Clock: 7
SUB_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SUB A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::sub(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: SBC A,r
// Clock: 4
SBC_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SBC A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::sbc(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: SBC A,n
// Clock: 7
SBC_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SBC A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::sbc(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: SBC A,(HL)
// Clock: 7
SBC_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SBC A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::sbc(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: AND A,r
// Clock: 4
AND_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " AND A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseAnd(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: AND A,n
// Clock: 7
AND_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " AND A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseAnd(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: AND A,(HL)
// Clock: 7
AND_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " AND A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::bitwiseAnd(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: OR A,r
// Clock: 4
OR_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " OR A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseOr(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: OR A,n
// Clock: 7
OR_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " OR A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseOr(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: OR A,(HL)
// Clock: 7
OR_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " OR A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::bitwiseOr(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: XOR A,r
// Clock: 4
XOR_A_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " XOR A,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseXor(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: XOR A,n
// Clock: 7
XOR_A_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " XOR A,n" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = ALU::bitwiseXor(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: XOR A,(HL)
// Clock: 7
XOR_A__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " XOR A,(HL)" << std::endl;
#endif


	m_reg.A = ALU::bitwiseXor(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: CP r
// Clock: 4
CP_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CP r" << " (r=" << r << ")" << std::endl;
#endif


	ALU::sub(m_reg.A, r, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: CP n
// Clock: 7
CP_n:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CP n" << " (n=" << n << ")" << std::endl;
#endif


	ALU::sub(m_reg.A, n, m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: CP (HL)
// Clock: 7
CP__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CP (HL)" << std::endl;
#endif


	ALU::sub(m_reg.A, m_mem.read8(m_reg.HL), m_reg.F);
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: INC r
// Clock: 4
INC_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 3) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " INC r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::add(r, 1, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: INC (HL)
// Clock: 11
INC__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " INC (HL)" << std::endl;
#endif


	m_mem.write8(m_reg.HL, ALU::add(m_mem.read8(m_reg.HL), 1, m_reg.F));
	clock += 11;

}
goto END_INSTRUCTIONS;


// Instruction: DEC r
// Clock: 4
DEC_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 3) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " DEC r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::sub(r, 1, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: DEC (HL)
// Clock: 11
DEC__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " DEC (HL)" << std::endl;
#endif


	m_mem.write8(m_reg.HL, ALU::sub(m_mem.read8(m_reg.HL), 1, m_reg.F));
	clock += 11;

}
goto END_INSTRUCTIONS;


// Instruction: DAA
// Clock: 4
DAA:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " DAA" << std::endl;
#endif


	m_reg.A = DAA(m_reg.A, m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: CPL
// Clock: 4
CPL:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CPL" << std::endl;
#endif


	m_reg.A = ~m_reg.A;
	m_reg.F.H = 1;
	m_reg.F.N = 1;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: CCF
// Clock: 4
CCF:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CCF" << std::endl;
#endif


	m_reg.F.C = (m_reg.F.C) ? 0 : 1;
	m_reg.F.N = 0;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: SCF
// Clock: 4
SCF:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SCF" << std::endl;
#endif


	m_reg.F.H = 0;
	m_reg.F.N = 0;
	m_reg.F.C = 1;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP
// Clock: 4
NOP:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP" << std::endl;
#endif


	// Do nothing
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: HALT
// Clock: 4
HALT:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " HALT" << std::endl;
#endif


	halt();
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: DI
// Clock: 4
DI:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " DI" << std::endl;
#endif


	m_iff1 = 0;
	m_iff2 = 0;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: EI
// Clock: 4
EI:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " EI" << std::endl;
#endif


	m_iff1 = 1;
	m_iff2 = 1;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: ADD HL, ss
// Clock: 11
ADD_HL__ss:
{
	DWORD& s(m_reg.begin16[regMask_s[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADD HL, ss" << " (s=" << s << ")" << std::endl;
#endif


	m_reg.HL = ALU::add(m_reg.HL, s, m_reg.F);
	clock += 11;

}
goto END_INSTRUCTIONS;


// Instruction: INC ss
// Clock: 6
INC_ss:
{
	DWORD& s(m_reg.begin16[regMask_s[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " INC ss" << " (s=" << s << ")" << std::endl;
#endif


	s.inc();
	clock += 6;

}
goto END_INSTRUCTIONS;


// Instruction: DEC ss
// Clock: 6
DEC_ss:
{
	DWORD& s(m_reg.begin16[regMask_s[(opcode >> 4) & (uint8_t(-1) >> (8 - 2)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " DEC ss" << " (s=" << s << ")" << std::endl;
#endif


	s.dec();
	clock += 6;

}
goto END_INSTRUCTIONS;


// Instruction: RLCA
// Clock: 4
RLCA:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RLCA" << std::endl;
#endif


	m_reg.A = ALU::rotateLeft(m_reg.A, m_reg.F, false);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: RLA
// Clock: 4
RLA:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RLA" << std::endl;
#endif


	m_reg.A = ALU::rotateLeft(m_reg.A, m_reg.F, true);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: RRCA
// Clock: 4
RRCA:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RRCA" << std::endl;
#endif


	m_reg.A = ALU::rotateRight(m_reg.A, m_reg.F, false);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: RRA
// Clock: 4
RRA:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RRA" << std::endl;
#endif


	m_reg.A = ALU::rotateRight(m_reg.A, m_reg.F, true);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: RLC r
// Clock: 8
RLC_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RLC r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::rotateLeft(r, m_reg.F, false, true);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: RLC (HL)
// Clock: 15
RLC__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RLC (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::rotateLeft(m_mem.read8(m_reg.HL), m_reg.F, false, true)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: RL r
// Clock: 8
RL_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RL r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::rotateLeft(r, m_reg.F, true, true);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: RL (HL)
// Clock: 15
RL__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RL (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::rotateLeft(m_mem.read8(m_reg.HL), m_reg.F, true, true)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: RRC r
// Clock: 8
RRC_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RRC r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::rotateRight(r, m_reg.F, false, true);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: RRC (HL)
// Clock: 15
RRC__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RRC (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::rotateRight(m_mem.read8(m_reg.HL), m_reg.F, false, true)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: RR r
// Clock: 8
RR_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RR r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::rotateRight(r, m_reg.F, true, true);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: RR (HL)
// Clock: 15
RR__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RR (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::rotateRight(m_mem.read8(m_reg.HL), m_reg.F, true, true)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: SLA r
// Clock: 8
SLA_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SLA r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::shiftLeft(r, m_reg.F);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: SLA (HL)
// Clock: 15
SLA__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SLA (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::shiftLeft(m_mem.read8(m_reg.HL), m_reg.F)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: SRA r
// Clock: 8
SRA_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SRA r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::shiftRight(r, m_reg.F, true);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: SRA (HL)
// Clock: 15
SRA__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SRA (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::shiftRight(m_mem.read8(m_reg.HL), m_reg.F, true)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: SRL r
// Clock: 8
SRL_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SRL r" << " (r=" << r << ")" << std::endl;
#endif


	r = ALU::shiftRight(r, m_reg.F, false);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: SRL (HL)
// Clock: 15
SRL__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SRL (HL)" << std::endl;
#endif


	m_mem.write8(
		m_reg.HL,
		ALU::shiftRight(m_mem.read8(m_reg.HL), m_reg.F, false)
		);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: BIT b,r
// Clock: 8
BIT_b_r:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " BIT b,r" << " (r=" << r << ")" << std::endl;
#endif


	m_reg.F.Z = ((~r) >> b) & 0x1;
	m_reg.F.H = 1;
	m_reg.F.N = 0;
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: BIT b,(HL)
// Clock: 12
BIT_b__HL_:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " BIT b,(HL)" << std::endl;
#endif


	m_reg.F.Z = ((~m_mem.read8(m_reg.HL)) >> b) & 0x1;
	m_reg.F.H = 1;
	m_reg.F.N = 0;
	clock += 12;

}
goto END_INSTRUCTIONS;


// Instruction: SET b,r
// Clock: 8
SET_b_r:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SET b,r" << " (r=" << r << ")" << std::endl;
#endif


	r |= (0x1 << b);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: SET b,(HL)
// Clock: 15
SET_b__HL_:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SET b,(HL)" << std::endl;
#endif


	uint8_t tmp(m_mem.read8(m_reg.HL));
	tmp |= (0x1 << b);
	m_mem.write8(m_reg.HL, tmp);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: RES b,r
// Clock: 8
RES_b_r:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RES b,r" << " (r=" << r << ")" << std::endl;
#endif


	r &= 0xFF ^ (0x1 << b);
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: RES b,(HL)
// Clock: 15
RES_b__HL_:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RES b,(HL)" << std::endl;
#endif


	uint8_t tmp(m_mem.read8(m_reg.HL));
	tmp &= 0xFF ^ (0x1 << b);
	m_mem.write8(m_reg.HL, tmp);
	clock += 15;

}
goto END_INSTRUCTIONS;


// Instruction: JP nn
// Clock: 10
JP_nn:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP nn" << " (N=" << N << ")" << std::endl;
#endif


	m_reg.PC = N;
	clock += 10;

}
goto END_INSTRUCTIONS;


// Instruction: JR e
// Clock: 12
JR_e:
{
	int8_t e(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JR e" << std::endl;
#endif


	uint16_t pc = m_reg.PC.host();
	pc = pc + e;
	m_reg.PC = htoz(pc);
	clock += 12;

}
goto END_INSTRUCTIONS;


// Instruction: JR C, e
// Clock: 7
JR_C__e:
{
	int8_t e(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JR C, e" << std::endl;
#endif


	if (m_reg.F.C)
	{
		uint16_t pc = m_reg.PC.host();
		pc = pc + e;
		m_reg.PC = htoz(pc);

		clock += 5;
	}
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: JR NC, e
// Clock: 7
JR_NC__e:
{
	int8_t e(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JR NC, e" << std::endl;
#endif


	if (!m_reg.F.C)
	{
		uint16_t pc = m_reg.PC.host();
		pc = pc + e;
		m_reg.PC = htoz(pc);

		clock += 5;
	}
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: JR Z, e
// Clock: 7
JR_Z__e:
{
	int8_t e(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JR Z, e" << std::endl;
#endif


	if (m_reg.F.Z)
	{
		uint16_t pc = m_reg.PC.host();
		pc = pc + e;
		m_reg.PC = htoz(pc);

		clock += 5;
	}
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: JR NZ, e
// Clock: 7
JR_NZ__e:
{
	int8_t e(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JR NZ, e" << std::endl;
#endif


	if (!m_reg.F.Z)
	{
		uint16_t pc = m_reg.PC.host();
		pc = pc + e;
		m_reg.PC = htoz(pc);

		clock += 5;
	}
	clock += 7;

}
goto END_INSTRUCTIONS;


// Instruction: JP (HL)
// Clock: 4
JP__HL_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP (HL)" << std::endl;
#endif


	m_reg.PC = m_reg.HL;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: CALL nn
// Clock: 17
CALL_nn:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CALL nn" << " (N=" << N << ")" << std::endl;
#endif


	m_reg.SP.dec(2);
	m_mem.write16(m_reg.SP, m_reg.PC);
	m_reg.PC = N;
	clock += 17;

}
goto END_INSTRUCTIONS;


// Instruction: RET
// Clock: 10
RET:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RET" << " (N=" << N << ")" << std::endl;
#endif


	m_reg.PC = m_mem.read16(m_reg.SP);
	m_reg.SP.inc(2);
	clock += 10;

}
goto END_INSTRUCTIONS;


// Instruction: RST p
// Clock: 11
RST_p:
{
uint8_t b((opcode >> 3) & (uint8_t(-1) >> (8 - 3)));
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RST p" << std::endl;
#endif


	uint8_t addr;
	switch (b)
	{
	case 0: addr = 0; break;
	case 1: addr = 0x08; break;
	case 2: addr = 0x10; break;
	case 3: addr = 0x18; break;
	case 4: addr = 0x20; break;
	case 5: addr = 0x28; break;
	case 6: addr = 0x30; break;
	case 7: addr = 0x38; break;
	}

	m_reg.SP.dec(2);
	m_mem.write16(m_reg.SP, m_reg.PC);
	m_reg.PC.h = 0;
	m_reg.PC.l = addr;
	clock += 11;

}
goto END_INSTRUCTIONS;


// Instruction: CB
// Clock: 0
CB:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CB" << std::endl;
#endif


	prefix = (prefix << 8) | opcode;
	opcode = op8();
	goto *(OpcodeTableCB[opcode]);
	clock += 0;

}
goto END_INSTRUCTIONS;


// Instruction: LD (nn),SP
// Clock: 16
LD__nn__SP:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (nn),SP" << " (N=" << N << ")" << std::endl;
#endif


	m_mem.write16(N, m_reg.SP);
	clock += 16;

}
goto END_INSTRUCTIONS;


// Instruction: LD (HLI),A
// Clock: 16
LD__HLI__A:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (HLI),A" << std::endl;
#endif


	m_mem.write8(m_reg.HL, m_reg.A);
	m_reg.HL.inc();
	clock += 16;

}
goto END_INSTRUCTIONS;


// Instruction: LD A, (HLI)
// Clock: 16
LD_A___HLI_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A, (HLI)" << std::endl;
#endif


	m_reg.A = m_mem.read8(m_reg.HL);
	m_reg.HL.inc();
	clock += 16;

}
goto END_INSTRUCTIONS;


// Instruction: LD (HLD),A
// Clock: 13
LD__HLD__A:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (HLD),A" << std::endl;
#endif


	m_mem.write8(m_reg.HL, m_reg.A);
	m_reg.HL.dec();
	clock += 13;

}
goto END_INSTRUCTIONS;


// Instruction: LD A,(HLD)
// Clock: 13
LD_A__HLD_:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A,(HLD)" << std::endl;
#endif


	m_reg.A = m_mem.read8(m_reg.HL);
	m_reg.HL.dec();
	clock += 13;

}
goto END_INSTRUCTIONS;


// Instruction: RETI
// Clock: 14
RETI:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RETI" << std::endl;
#endif


	m_reg.PC = m_mem.read16(m_reg.SP);
	m_reg.SP.inc(2);
	m_iff1 = 1;
	m_iff2 = 1;
	clock += 14;

}
goto END_INSTRUCTIONS;


// Instruction: STOP
// Clock: 4
STOP:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " STOP" << std::endl;
#endif


	halt();
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: ADD SP,d
// Clock: 4
ADD_SP_d:
{
	int8_t d(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " ADD SP,d" << std::endl;
#endif


	m_reg.SP = ALU::add(m_reg.SP, htoz(int16_t(d)), m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LD (N),A
// Clock: 4
LD__N__A:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (N),A" << " (N=" << N << ")" << std::endl;
#endif


	m_mem.write8(N, m_reg.A);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LDHL SP,d
// Clock: 4
LDHL_SP_d:
{
	int8_t d(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LDHL SP,d" << std::endl;
#endif


	m_reg.HL = ALU::add(m_reg.SP, htoz(int16_t(d)), m_reg.F);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LD A,(N)
// Clock: 4
LD_A__N_:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A,(N)" << " (N=" << N << ")" << std::endl;
#endif


	m_reg.A = m_mem.read8(N);;
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: RET C
// Clock: 5
RET_C:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RET C" << std::endl;
#endif


	if (m_reg.F.C)
	{
		m_reg.PC = m_mem.read16(m_reg.SP);
		m_reg.SP.inc(2);
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: RET NC
// Clock: 5
RET_NC:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RET NC" << std::endl;
#endif


	if (!m_reg.F.C)
	{
		m_reg.PC = m_mem.read16(m_reg.SP);
		m_reg.SP.inc(2);
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: JP Z
// Clock: 5
JP_Z:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP Z" << " (N=" << N << ")" << std::endl;
#endif


	if (m_reg.F.Z)
	{
		m_reg.PC = N;
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: RET Z
// Clock: 5
RET_Z:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RET Z" << std::endl;
#endif


	if (m_reg.F.Z)
	{
		m_reg.PC = m_mem.read16(m_reg.SP);
		m_reg.SP.inc(2);
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: RET NZ
// Clock: 8
RET_NZ:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " RET NZ" << std::endl;
#endif


	if (!m_reg.F.Z)
	{
		m_reg.PC = m_mem.read16(m_reg.SP);
		m_reg.SP.inc(2);
		clock += 6;
	}
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: CALL Z
// Clock: 8
CALL_Z:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CALL Z" << " (N=" << N << ")" << std::endl;
#endif


	if (m_reg.F.Z)
	{
		m_reg.SP.dec(2);
		m_mem.write16(m_reg.SP, m_reg.PC);
		m_reg.PC = N;
		clock += 7;
	}
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: CALL NZ
// Clock: 8
CALL_NZ:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CALL NZ" << " (N=" << N << ")" << std::endl;
#endif


	if (!m_reg.F.Z)
	{
		m_reg.SP.dec(2);
		m_mem.write16(m_reg.SP, m_reg.PC);
		m_reg.PC = N;
		clock += 7;
	}
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: CALL C
// Clock: 8
CALL_C:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CALL C" << " (N=" << N << ")" << std::endl;
#endif


	if (m_reg.F.C)
	{
		m_reg.SP.dec(2);
		m_mem.write16(m_reg.SP, m_reg.PC);
		m_reg.PC = N;
		clock += 7;
	}
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: CALL NC
// Clock: 8
CALL_NC:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " CALL NC" << " (N=" << N << ")" << std::endl;
#endif


	if (!m_reg.F.C)
	{
		m_reg.SP.dec(2);
		m_mem.write16(m_reg.SP, m_reg.PC);
		m_reg.PC = N;
		clock += 7;
	}
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: JP NZ
// Clock: 5
JP_NZ:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP NZ" << " (N=" << N << ")" << std::endl;
#endif


	if (!m_reg.F.Z)
	{
		m_reg.PC = N;
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: JP NC
// Clock: 5
JP_NC:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP NC" << " (N=" << N << ")" << std::endl;
#endif


	if (!m_reg.F.C)
	{
		m_reg.PC = N;
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: JP C
// Clock: 5
JP_C:
{
	DWORD N(op16());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " JP C" << " (N=" << N << ")" << std::endl;
#endif


	if (m_reg.F.C)
	{
		m_reg.PC = N;
		clock += 6;
	}
	clock += 5;

}
goto END_INSTRUCTIONS;


// Instruction: SWAP r
// Clock: 8
SWAP_r:
{
	uint8_t& r(m_reg.begin8[regMask_r[(opcode >> 0) & (uint8_t(-1) >> (8 - 3)) ]]);
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " SWAP r" << " (r=" << r << ")" << std::endl;
#endif


	uint8_t tmp(r);
	r <<= 4;
	r |= (tmp >> 4);

	m_reg.F.Z = r == 0;
	m_reg.F.N = 0;
	m_reg.F.H = 0;
	m_reg.F.C = 0;
	clock += 8;

}
goto END_INSTRUCTIONS;


// Instruction: LD (FF00+n),A
// Clock: 4
LD__FF00_n__A:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (FF00+n),A" << " (n=" << n << ")" << std::endl;
#endif


	m_mem.write8(htoz(0xFF00 + n), m_reg.A);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LD (FF00+C),A
// Clock: 4
LD__FF00_C__A:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD (FF00+C),A" << std::endl;
#endif


	m_mem.write8(htoz(0xFF00 + m_reg.C), m_reg.A);
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: LD A,(FF00+d)
// Clock: 4
LD_A__FF00_d_:
{
	uint8_t n(op8());
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " LD A,(FF00+d)" << " (n=" << n << ")" << std::endl;
#endif


	m_reg.A = m_mem.read8(htoz(0xFF00 + n));
	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_D3
// Clock: 4
NOP_D3:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_D3" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_DB
// Clock: 4
NOP_DB:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_DB" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_DD
// Clock: 4
NOP_DD:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_DD" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_E3
// Clock: 4
NOP_E3:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_E3" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_E4
// Clock: 4
NOP_E4:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_E4" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_EB
// Clock: 4
NOP_EB:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_EB" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_EC
// Clock: 4
NOP_EC:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_EC" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_F2
// Clock: 4
NOP_F2:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_F2" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_F4
// Clock: 4
NOP_F4:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_F4" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_FC
// Clock: 4
NOP_FC:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_FC" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


// Instruction: NOP_FD
// Clock: 4
NOP_FD:
{
#ifdef GLBOY_DEBUG
std::ostream& trace(log(Log::DBG_TRACE));
trace << opcodePC << ": ";
if (prefix) trace << hex(prefix) << " ";
trace << hex(opcode) << " NOP_FD" << std::endl;
#endif


	clock += 4;

}
goto END_INSTRUCTIONS;


