//	Copyright (C) 2011 Michael McMaster <michael@codesrc.com>
//
//	This file is part of glBoy.
//
//	glBoy is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	glBoy is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with glBoy.  If not, see <http://www.gnu.org/licenses/>.

#ifndef GLBOY_MEMORYMAP_HH
#define GLBOY_MEMORYMAP_HH

#include "glBoy.hh"
#include "DWORD.hh"

#include <memory>
#include <set>
#include <vector>
namespace glBoy
{

class MemoryMap
{
public:
	class Memory
	{
	public:
		virtual ~Memory() {}

		virtual uint8_t read8(uint16_t address) = 0;
		virtual void write8(uint16_t address, uint8_t value) = 0;
		virtual void copy(uint8_t* dest, uint16_t address, int bytes);
	};

	MemoryMap();

	void map(
		uint16_t base,
		uint16_t size,
		const std::shared_ptr<Memory>& mem
		);

	void remove(uint16_t base);

	uint8_t read8(DWORD address, int8_t offset = 0);
	void write8(DWORD address, uint8_t value);
	void write8(DWORD address, int8_t offset, uint8_t value);

	DWORD read16(DWORD address, int8_t offset = 0);
	void write16(DWORD address, DWORD value);
	void write16(DWORD address, int8_t offset, DWORD value);

	void copy(uint8_t* dest, uint16_t srcAddress, int bytes);

private:
	struct MappedArea
	{
		MappedArea(
			uint16_t _base, uint16_t _size, std::shared_ptr<Memory> _mem
			) :
			base(_base), size(_size), memory(_mem) {}

		uint16_t base;
		uint16_t size;
		std::shared_ptr<Memory> memory;

		bool operator<(const MappedArea& b) const { return base < b.base; }
	};
	const MappedArea& get(uint16_t address) const;

	typedef std::vector<MappedArea> MapType;
	MapType m_map;
	//std::set<MappedArea> m_map;
};

} // namespace glBoy

#endif

